#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <richedit.h>
#include "AkelEdit.h"
#include "AkelDLL.h"
#include "Resources\Resource.h"


//Defines
#define STRID_UPDATEINTERVAL 1
#define STRID_MS             2
#define STRID_COMPLETEREOPEN 3
#define STRID_SCROLLEND      4
#define STRID_OK             5
#define STRID_CANCEL         6

#ifndef CP_UNICODE_UCS2_LE
  #define CP_UNICODE_UCS2_LE 1200
#endif
#ifndef CP_UNICODE_UCS2_BE
  #define CP_UNICODE_UCS2_BE 1201
#endif

//Functions prototypes
BOOL CALLBACK SetupDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK SetupDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);
void ChangeByteOrder(unsigned char *lpBuffer, unsigned int nBufferLen);
BOOL GetFileWriteTimeA(HANDLE hFile, char *pFile, FILETIME *ft);
BOOL GetFileWriteTimeW(HANDLE hFile, wchar_t *wpFile, FILETIME *ft);
void ReadOptionsA();
void ReadOptionsW();
void SaveOptionsA();
void SaveOptionsW();
char* GetLangStringA(LANGID wLangID, int nStringID);
wchar_t* GetLangStringW(LANGID wLangID, int nStringID);
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();

//Global variables
char szPluginName[MAX_PATH];
wchar_t wszPluginName[MAX_PATH];
HINSTANCE hInstanceDLL;
HWND hMainWnd;
HWND hWndEdit;
HWND hMdiClient;
HICON hMainIcon;
BOOL bOldWindows;
BOOL bAkelEdit;
BOOL bMDI;
LANGID wLangSystem;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
FILETIME ftCurrent={0};
DWORD dwCurrentPointer=0;
DWORD dwUpdateTimer=0;
DWORD dwUpdateInterval=1000;
BOOL bCompleteReopen=FALSE;
BOOL bScrollEnd=TRUE;
WNDPROCDATA *NewMainProcData=NULL;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="LogView";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Settings(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=FALSE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  if (bOldWindows)
    DialogBoxA(hInstanceDLL, MAKEINTRESOURCEA(IDD_SETUP), hMainWnd, (DLGPROC)SetupDlgProcA);
  else
    DialogBoxW(hInstanceDLL, MAKEINTRESOURCEW(IDD_SETUP), hMainWnd, (DLGPROC)SetupDlgProcW);

  //If plugin already loaded, stay in memory, but show as non-active
  if (pd->bInMemory) pd->nUnload=UD_NONUNLOAD_NONACTIVE;
}

extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  if (bInitMain)
  {
    UninitMain();
    hWndEdit=NULL;
  }
  else
  {
    hWndEdit=pd->hWndEdit;
    InitMain();

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

BOOL CALLBACK SetupDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndUpdateInterval;
  static HWND hWndCompleteReopenCheck;
  static HWND hWndScrollEndCheck;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndUpdateInterval=GetDlgItem(hDlg, IDC_UPDATEINTERVAL);
    hWndCompleteReopenCheck=GetDlgItem(hDlg, IDC_COMPLETEREOPEN_CHECK);
    hWndScrollEndCheck=GetDlgItem(hDlg, IDC_SCROLLEND_CHECK);

    SetDlgItemTextA(hDlg, IDC_UPDATEINTERVAL_LABEL, GetLangStringA(wLangSystem, STRID_UPDATEINTERVAL));
    SetDlgItemTextA(hDlg, IDC_MS_LABEL, GetLangStringA(wLangSystem, STRID_MS));
    SetDlgItemTextA(hDlg, IDC_COMPLETEREOPEN_CHECK, GetLangStringA(wLangSystem, STRID_COMPLETEREOPEN));
    SetDlgItemTextA(hDlg, IDC_SCROLLEND_CHECK, GetLangStringA(wLangSystem, STRID_SCROLLEND));
    SetDlgItemTextA(hDlg, IDOK, GetLangStringA(wLangSystem, STRID_OK));
    SetDlgItemTextA(hDlg, IDCANCEL, GetLangStringA(wLangSystem, STRID_CANCEL));

    SendMessage(hWndUpdateInterval, EM_LIMITTEXT, 6, 0);
    SetDlgItemInt(hDlg, IDC_UPDATEINTERVAL, dwUpdateInterval, FALSE);
    if (bCompleteReopen) SendMessage(hWndCompleteReopenCheck, BM_SETCHECK, BST_CHECKED, 0);
    if (!bAkelEdit || bScrollEnd) SendMessage(hWndScrollEndCheck, BM_SETCHECK, BST_CHECKED, 0);
    if (!bAkelEdit) EnableWindow(hWndScrollEndCheck, FALSE);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDOK)
    {
      dwUpdateInterval=GetDlgItemInt(hDlg, IDC_UPDATEINTERVAL, NULL, FALSE);
      bCompleteReopen=SendMessage(hWndCompleteReopenCheck, BM_GETCHECK, 0, 0);
      bScrollEnd=SendMessage(hWndScrollEndCheck, BM_GETCHECK, 0, 0);

      SaveOptionsA();
      EndDialog(hDlg, 0);

      if (bInitMain)
      {
        UninitMain();
        InitMain();
      }
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

BOOL CALLBACK SetupDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndUpdateInterval;
  static HWND hWndCompleteReopenCheck;
  static HWND hWndScrollEndCheck;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndUpdateInterval=GetDlgItem(hDlg, IDC_UPDATEINTERVAL);
    hWndCompleteReopenCheck=GetDlgItem(hDlg, IDC_COMPLETEREOPEN_CHECK);
    hWndScrollEndCheck=GetDlgItem(hDlg, IDC_SCROLLEND_CHECK);

    SetDlgItemTextW(hDlg, IDC_UPDATEINTERVAL_LABEL, GetLangStringW(wLangSystem, STRID_UPDATEINTERVAL));
    SetDlgItemTextW(hDlg, IDC_MS_LABEL, GetLangStringW(wLangSystem, STRID_MS));
    SetDlgItemTextW(hDlg, IDC_COMPLETEREOPEN_CHECK, GetLangStringW(wLangSystem, STRID_COMPLETEREOPEN));
    SetDlgItemTextW(hDlg, IDC_SCROLLEND_CHECK, GetLangStringW(wLangSystem, STRID_SCROLLEND));
    SetDlgItemTextW(hDlg, IDOK, GetLangStringW(wLangSystem, STRID_OK));
    SetDlgItemTextW(hDlg, IDCANCEL, GetLangStringW(wLangSystem, STRID_CANCEL));

    SendMessage(hWndUpdateInterval, EM_LIMITTEXT, 6, 0);
    SetDlgItemInt(hDlg, IDC_UPDATEINTERVAL, dwUpdateInterval, FALSE);
    if (bCompleteReopen) SendMessage(hWndCompleteReopenCheck, BM_SETCHECK, BST_CHECKED, 0);
    if (!bAkelEdit || bScrollEnd) SendMessage(hWndScrollEndCheck, BM_SETCHECK, BST_CHECKED, 0);
    if (!bAkelEdit) EnableWindow(hWndScrollEndCheck, FALSE);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDOK)
    {
      dwUpdateInterval=GetDlgItemInt(hDlg, IDC_UPDATEINTERVAL, NULL, FALSE);
      bCompleteReopen=SendMessage(hWndCompleteReopenCheck, BM_GETCHECK, 0, 0);
      bScrollEnd=SendMessage(hWndScrollEndCheck, BM_GETCHECK, 0, 0);

      SaveOptionsW();
      EndDialog(hDlg, 0);

      if (bInitMain)
      {
        UninitMain();
        InitMain();
      }
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == AKDN_EDIT_ONSTART)
  {
    if (hWndEdit == NULL)
    {
      hWndEdit=(HWND)wParam;
    }
  }
  else if (uMsg == AKDN_EDIT_ONFINISH)
  {
    if (hWndEdit == (HWND)wParam)
    {
      hWndEdit=NULL;
      PostMessage(hMainWnd, AKD_DLLUNLOAD, (WPARAM)hInstanceDLL, (LPARAM)NULL);
    }
  }

  //Call next procedure
  return NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
{
  EDITINFO ei;
  FILETIME ftNew;

  if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei))
  {
    if (ei.hWndEdit == hWndEdit)
    {
      if (bOldWindows)
        GetFileWriteTimeA(NULL, (char *)ei.pFile, &ftNew);
      else
        GetFileWriteTimeW(NULL, (wchar_t *)ei.pFile, &ftNew);

      if (CompareFileTime(&ftCurrent, &ftNew))
      {
        CHARRANGE cr;
        HANDLE hFile;
        DWORD dwBlockSize;
        DWORD dwBytesRead;
        DWORD dwNewPointer=0;
        DWORD dwUnicodeBytes;
        unsigned char *pText;
        wchar_t *wpText;

        ftCurrent=ftNew;

        if (bCompleteReopen)
        {
          SendMessage(hMainWnd, WM_COMMAND, IDM_FILE_REOPEN, 0);
          if (bScrollEnd) SendMessage(ei.hWndEdit, EM_SETSEL, (WPARAM)-1, (LPARAM)-1);
        }
        else
        {
          if (bOldWindows)
            hFile=CreateFileA((char *)ei.pFile, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN, NULL);
          else
            hFile=CreateFileW((wchar_t *)ei.pFile, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN, NULL);

          if (hFile != INVALID_HANDLE_VALUE)
          {
            if ((dwNewPointer=GetFileSize(hFile, NULL)) != INVALID_FILE_SIZE)
            {
              if (dwNewPointer < dwCurrentPointer)
                dwCurrentPointer=0;

              dwBlockSize=dwNewPointer - dwCurrentPointer;

              if (pText=(unsigned char *)GlobalAlloc(GPTR, dwBlockSize + 2))
              {
                SetFilePointer(hFile, dwCurrentPointer, NULL, FILE_BEGIN);

                if (ReadFile(hFile, pText, dwBlockSize, &dwBytesRead, NULL))
                {
                  if (ei.nCodePage == CP_UNICODE_UCS2_LE)
                  {
                    wpText=(wchar_t *)pText;
                    dwUnicodeBytes=dwBytesRead;
                  }
                  else if (ei.nCodePage == CP_UNICODE_UCS2_BE)
                  {
                    ChangeByteOrder((unsigned char *)pText, dwBytesRead);
                    wpText=(wchar_t *)pText;
                    dwUnicodeBytes=dwBytesRead;
                  }
                  else
                  {
                    dwUnicodeBytes=MultiByteToWideChar(ei.nCodePage, 0, (char *)pText, dwBytesRead, NULL, 0) * sizeof(wchar_t);

                    if (wpText=(wchar_t *)GlobalAlloc(GPTR, dwUnicodeBytes + 2))
                    {
                      MultiByteToWideChar(ei.nCodePage, 0, (char *)pText, dwBytesRead, wpText, dwUnicodeBytes / sizeof(wchar_t));
                    }
                  }
                  wpText[dwUnicodeBytes / sizeof(wchar_t)]='\0';

                  if (!dwCurrentPointer)
                  {
                    SendMessage(ei.hWndEdit, EM_EXGETSEL, 0, (LPARAM)&cr);
                    SendMessage(ei.hWndEdit, WM_SETTEXT, 0, (LPARAM)L"");
                  }

                  //Add text
                  if (bAkelEdit)
                  {
                    AEAPPENDTEXTW at;

                    at.pText=wpText;
                    at.dwTextLen=dwUnicodeBytes / sizeof(wchar_t);
                    at.bColumnSel=FALSE;

                    SendMessage(ei.hWndEdit, AEM_LOCKCOLLECTUNDO, TRUE, 0);
                    SendMessage(ei.hWndEdit, AEM_APPENDTEXTW, 0, (LPARAM)&at);
                    SendMessage(ei.hWndEdit, AEM_LOCKCOLLECTUNDO, FALSE, 0);
                  }
                  else
                  {
                    SendMessage(ei.hWndEdit, EM_SETSEL, (WPARAM)-1, (LPARAM)-1);
                    SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)ei.hWndEdit, (LPARAM)wpText);
                  }
                  if (!ei.bModified) SendMessage(hMainWnd, AKD_SETMODIFY, (WPARAM)ei.hWndEdit, FALSE);

                  if (!dwCurrentPointer)
                  {
                    SendMessage(ei.hWndEdit, EM_EXSETSEL, 0, (LPARAM)&cr);
                  }
                  if (bScrollEnd) SendMessage(ei.hWndEdit, EM_SETSEL, (WPARAM)-1, (LPARAM)-1);
                }
                if (pText) GlobalFree((HGLOBAL)pText);
              }
              dwCurrentPointer=dwNewPointer;
            }
            CloseHandle(hFile);
          }
        }
      }
    }
  }
}

void ChangeByteOrder(unsigned char *lpBuffer, unsigned int nBufferLen)
{
  unsigned char ch;
  unsigned int a;
  unsigned int b;

  for (a=0, b=1; b < nBufferLen; a+=2, b+=2)
  {
    ch=lpBuffer[a];
    lpBuffer[a]=lpBuffer[b];
    lpBuffer[b]=ch;
  }
}

BOOL GetFileWriteTimeA(HANDLE hFile, char *pFile, FILETIME *ft)
{
  if (!hFile)
  {
    if ((hFile=CreateFileA(pFile, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE)
      return FALSE;
  }
  GetFileTime(hFile, NULL, NULL, ft);
  CloseHandle(hFile);
  return TRUE;
}

BOOL GetFileWriteTimeW(HANDLE hFile, wchar_t *wpFile, FILETIME *ft)
{
  if (!hFile)
  {
    if ((hFile=CreateFileW(wpFile, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE)
      return FALSE;
  }
  GetFileTime(hFile, NULL, NULL, ft);
  CloseHandle(hFile);
  return TRUE;
}


//// Options

void ReadOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)szPluginName))
  {
    po.pOptionName="UpdateInterval";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwUpdateInterval;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="CompleteReopenEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bCompleteReopen;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="ScrollEndEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bScrollEnd;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void ReadOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"UpdateInterval";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwUpdateInterval;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"CompleteReopenEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bCompleteReopen;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"ScrollEndEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bScrollEnd;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)szPluginName))
  {
    po.pOptionName="UpdateInterval";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwUpdateInterval;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="CompleteReopenEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bCompleteReopen;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="ScrollEndEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bScrollEnd;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"UpdateInterval";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&dwUpdateInterval;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"CompleteReopenEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bCompleteReopen;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"ScrollEndEnable";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&bScrollEnd;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

char* GetLangStringA(LANGID wLangID, int nStringID)
{
  static char szStringBuf[MAX_PATH];

  WideCharToMultiByte(CP_ACP, 0, GetLangStringW(wLangID, nStringID), -1, szStringBuf, MAX_PATH, NULL, NULL);
  return szStringBuf;
}

wchar_t* GetLangStringW(LANGID wLangID, int nStringID)
{
  if (wLangID == LANG_RUSSIAN)
  {
    if (nStringID == STRID_UPDATEINTERVAL)
      return L"\x0418\x043D\x0442\x0435\x0440\x0432\x0430\x043B\x0020\x043E\x0431\x043D\x043E\x0432\x043B\x0435\x043D\x0438\x044F";
    if (nStringID == STRID_MS)
      return L"\x043C\x0441\x002E";
    if (nStringID == STRID_COMPLETEREOPEN)
      return L"\x041F\x0435\x0440\x0435\x043E\x0442\x043A\x0440\x044B\x0432\x0430\x0442\x044C\x0020\x043F\x043E\x043B\x043D\x043E\x0441\x0442\x044C\x044E";
    if (nStringID == STRID_SCROLLEND)
      return L"\x041F\x0440\x043E\x043A\x0440\x0443\x0447\x0438\x0432\x0430\x0442\x044C\x0020\x0432\x0020\x043A\x043E\x043D\x0435\x0446";
    if (nStringID == STRID_OK)
      return L"\x004F\x004B";
    if (nStringID == STRID_CANCEL)
      return L"\x041E\x0442\x043C\x0435\x043D\x0430";
  }
  else
  {
    if (nStringID == STRID_UPDATEINTERVAL)
      return L"Update interval";
    if (nStringID == STRID_MS)
      return L"ms.";
    if (nStringID == STRID_COMPLETEREOPEN)
      return L"Complete reopening";
    if (nStringID == STRID_SCROLLEND)
      return L"Scroll to the end";
    if (nStringID == STRID_OK)
      return L"OK";
    if (nStringID == STRID_CANCEL)
      return L"Cancel";
  }
  return L"";
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hInstanceDLL=pd->hInstanceDLL;
  hMainWnd=pd->hMainWnd;
  hMdiClient=pd->hMdiClient;
  hMainIcon=pd->hMainIcon;
  bOldWindows=pd->bOldWindows;
  bAkelEdit=pd->bAkelEdit;
  bMDI=pd->bMDI;
  wLangSystem=PRIMARYLANGID(pd->wLangSystem);

  if (bOldWindows)
  {
    int i;

    for (i=0; ((char *)pd->pFunction)[i] != ':'; ++i)
      szPluginName[i]=((char *)pd->pFunction)[i];
    szPluginName[i]='\0';
  }
  else
  {
    int i;

    for (i=0; ((wchar_t *)pd->pFunction)[i] != ':'; ++i)
      wszPluginName[i]=((wchar_t *)pd->pFunction)[i];
    wszPluginName[i]='\0';
  }

  if (bOldWindows)
    ReadOptionsA();
  else
    ReadOptionsW();
}

void InitMain()
{
  bInitMain=TRUE;

  //Set timer
  dwUpdateTimer=SetTimer(NULL, 0, dwUpdateInterval, (TIMERPROC)TimerProc);

  //SubClass main procedure
  NewMainProcData=NULL;
  SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NewMainProc, (LPARAM)&NewMainProcData);
}

void UninitMain()
{
  bInitMain=FALSE;

  //Remove subclass
  if (NewMainProcData)
  {
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NULL, (LPARAM)&NewMainProcData);
    NewMainProcData=NULL;
  }

  //Remove timer
  if (dwUpdateTimer)
  {
    KillTimer(NULL, dwUpdateTimer);
    dwUpdateTimer=0;
  }
  ftCurrent.dwLowDateTime=0;
  ftCurrent.dwHighDateTime=0;
  dwCurrentPointer=0;
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
    if (bInitMain) UninitMain();
  }
  return TRUE;
}
